@@ -21,6 +21,7 @@
 #include "transform.h"
 #include "imports.h"
 #include "keys.h"
+#include "security.h"
 
 #ifdef LIBXML_XINCLUDE_ENABLED
 #include <libxml/xinclude.h>
@@ -51,8 +52,7 @@ xsltNewDocument(xsltTransformContextPtr ctxt, xmlDocPtr doc) {
 
     cur = (xsltDocumentPtr) xmlMalloc(sizeof(xsltDocument));
     if (cur == NULL) {
-	xsltPrintErrorContext(ctxt, NULL, (xmlNodePtr) doc);
-        xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(ctxt, NULL, (xmlNodePtr) doc,
 		"xsltNewDocument : malloc failed\n");
 	return(NULL);
     }
@@ -79,8 +79,7 @@ xsltNewStyleDocument(xsltStylesheetPtr style, xmlDocPtr doc) {
 
     cur = (xsltDocumentPtr) xmlMalloc(sizeof(xsltDocument));
     if (cur == NULL) {
-	xsltPrintErrorContext(NULL, style, (xmlNodePtr) doc);
-        xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(NULL, style, (xmlNodePtr) doc,
 		"xsltNewStyleDocument : malloc failed\n");
 	return(NULL);
     }
@@ -163,6 +162,21 @@ xsltLoadDocument(xsltTransformContextPtr ctxt, const xmlChar *URI) {
     if ((ctxt == NULL) || (URI == NULL))
 	return(NULL);
 
+    /*
+     * Security framework check
+     */
+    if (ctxt->sec != NULL) {
+	int res;
+	
+	res = xsltCheckRead(ctxt->sec, ctxt, URI);
+	if (res == 0) {
+	    xsltTransformError(ctxt, NULL, NULL,
+		 "xsltLoadDocument: read rights for %s denied\n",
+			     URI);
+	    return(NULL);
+	}
+    }
+
     /*
      * Walk the context list to find the document if preparsed
      */
@@ -182,8 +196,7 @@ xsltLoadDocument(xsltTransformContextPtr ctxt, const xmlChar *URI) {
 #ifdef LIBXML_XINCLUDE_ENABLED
 	xmlXIncludeProcess(doc);
 #else
-	xsltPrintErrorContext(ctxt, NULL, NULL);
-        xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(ctxt, NULL, NULL,
 	    "xsltLoadDocument(%s) : XInclude processing not compiled in\n",
 	                 URI);
 #endif
@@ -211,10 +224,27 @@ xsltDocumentPtr
 xsltLoadStyleDocument(xsltStylesheetPtr style, const xmlChar *URI) {
     xsltDocumentPtr ret;
     xmlDocPtr doc;
+    xsltSecurityPrefsPtr sec;
 
     if ((style == NULL) || (URI == NULL))
 	return(NULL);
 
+    /*
+     * Security framework check
+     */
+    sec = xsltGetDefaultSecurityPrefs();
+    if (sec != NULL) {
+	int res;
+
+	res = xsltCheckRead(sec, NULL, URI);
+	if (res == 0) {
+	    xsltTransformError(NULL, NULL, NULL,
+		 "xsltLoadStyleDocument: read rights for %s denied\n",
+			     URI);
+	    return(NULL);
+	}
+    }
+
     /*
      * Walk the context list to find the document if preparsed
      */
