@@ -56,14 +56,14 @@
 #include <libexslt/exsltconfig.h>
 
 #if defined(WIN32) && !defined (__CYGWIN__)
-#ifdef _MSC_VER
+#if defined(_MSC_VER) || defined(__MINGW32__)
 #include <winsock2.h>
-#pragma comment(lib, "ws2_32.lib")
 #define gettimeofday(p1,p2)
 #define HAVE_TIME_H
 #include <time.h>
 #define HAVE_STDARG_H
 #include <stdarg.h>
+#define snprintf _snprintf
 #endif /* _MS_VER */
 #else /* WIN32 */
 #if defined(HAVE_SYS_TIME_H)
@@ -172,12 +172,11 @@ xsltprocExternalEntityLoader(const char *URL, const char *ID,
     }
     for (i = 0;i < nbpaths;i++) {
 	xmlChar *newURL;
-	int len;
 
-	len = xmlStrlen(paths[i]) + xmlStrlen(BAD_CAST URL) + 5;
-	newURL = xmlMalloc(len);
+	newURL = xmlStrdup((const xmlChar *) paths[i]);
+	newURL = xmlStrcat(newURL, (const xmlChar *) "/");
+	newURL = xmlStrcat(newURL, (const xmlChar *) URL);
 	if (newURL != NULL) {
-	    snprintf(newURL, len, "%s/%s", paths[i], URL);
 	    ret = defaultEntityLoader((const char *)newURL, ID, ctxt);
 	    xmlFree(newURL);
 	    if (ret != NULL) {
